# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

"""
Utilities used by the various APIs.
"""

from functools import wraps
import re

from yampy.models import extract_id


def instance_replacer(*types):
    """
    Decorator for functions that passes all key/value pairs where the value
    has one of the given types to the decorated function. The decorated
    function should return a dict, which will be used as a replacement for
    the key/value pair that was pass to it.

    For example:
      @instance_replacer(bool)
      def boolean_inverter(key, value):
          return {"not_%s" % key: not value}

      boolean_inverter({"yes": True, "no": False, "other": 1})
      {"not_yes": False, "not_no": True, "other": 1}
    """
    def decorator(func):
        @wraps(func)
        def inner(arguments):
            result = {}
            for k, v in arguments.items():
                if isinstance(v, types):
                    result.update(func(k, v))
                else:
                    result[k] = v
            return result
        return inner
    return decorator


@instance_replacer(dict)
def flatten_dicts(prefix, collection):
    """
    Flattens nested dictionaries, joining the keys with an underscore.
    e.g. {"foo": {"bar": 1}}  becomes  {"foo_bar": 1}
    """
    result = {}
    for key, value in collection.items():
        item_key = "%s_%s" % (prefix, key)
        result[item_key] = value
    return result


@instance_replacer(list, tuple)
def flatten_lists(prefix, collection):
    """
    Expands all lists and tuples in a dictionary, so that each list item has
    its own key.
    e.g. {"foo": ("a", "b")}  becomes  {"foo1": "a", "foo2": "b"}
    """
    result = {}
    for index, value in enumerate(collection):
        item_key = "%s%d" % (prefix, index + 1)
        result[item_key] = value
    return result


@instance_replacer(bool)
def stringify_booleans(key, value):
    """
    Replaces all boolean values in a dictionary with lowercase strings.
    e.g. {"yes": True}  becomes  {"yes": "true"}
    """
    if value:
        return {key: "true"}
    else:
        return {key: "false"}


@instance_replacer(type(None))
def none_filter(key, value):
    """
    Removes None values from a dictionary.
    e.g. {"number": 1, "none": None}  becomes  {"number": 1}
    """
    return {}


class IDExtractor(object):
    """
    A callable object which, when called with a dict, will attempt to extract
    an ID from each value whose key matches the regular expression provided to
    the constructor.
    """

    def __init__(self, pattern=r'^.*_id$'):
        self._key_matcher = re.compile(pattern)

    def __call__(self, arguments):
        """
        Attempts to extract an ID from the value of any key that matches the
        pattern that was provided to the initialiser.
        e.g. With the default pattern of '^.*_id$'
             {"foo_id": {"id": 3}}  becomes  {"foo_id": 3}
        """
        result = arguments.copy()
        for key in arguments:
            if self._key_matcher.match(key):
                result[key] = extract_id(arguments[key])
        return result


class ArgumentConverter(object):
    """
    A callable object that takes a dict (in the form of keyword arguments),
    passes it through various converters and returns the result.
    """

    def __init__(self, *converters):
        """
        Initialize an ArgumentConverter that will call each of the given
        converters in turn. Converters should be callable, take a dict, and
        return a dict.
        """
        self._converters = converters

    def __call__(self, **kwargs):
        """
        Passes the dict of keyword arguments through each converter in turn,
        and returns the result.
        """
        converted_args = kwargs.copy()
        for converter in self._converters:
            converted_args = converter(converted_args)
        return converted_args



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from yampy.errors import InvalidOpenGraphObjectError, TooManyTopicsError
from yampy.apis.utils import ArgumentConverter, IDExtractor, flatten_lists, \
                             flatten_dicts, stringify_booleans, none_filter
from yampy.models import extract_id


class MessagesAPI(object):
    """
    Provides an interface for accessing the message related endpoints of the
    Yammer API. You should not instantiate this class directly; use the
    :meth:`yampy.Yammer.messages` method instead.
    """

    def __init__(self, client):
        """
        Initializes a new MessagesAPI that will use the given ``client`` object
        to make HTTP requests.
        """
        self._client = client
        self._argument_converter = ArgumentConverter(
            IDExtractor(r"^(older|newer)_than|.*_id$"),
            flatten_lists,
            flatten_dicts,
            stringify_booleans,
            none_filter,
        )

    def all(self, older_than=None, newer_than=None,
            limit=None, threaded=None):
        """
        Returns public messages from the current user's network.

        Customize the response using the keyword arguments:

        * ``older_than`` -- Only fetch messages older than this message ID.
        * ``newer_than`` -- Only fetch messages newer than this message ID.
        * ``limit`` -- Only fetch this many messages.
        * ``threaded`` -- Set to ``True`` to only receive the first message of
          each thread, or to ``"extended"`` to recieve the first and two newest
          messages from each thread.
        """
        return self._client.get("/messages", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def from_my_feed(self, older_than=None, newer_than=None,
                     limit=None, threaded=None):
        """
        Returns messages from the current user's feed. This will either
        correspond to :meth:`from_top_conversations` or
        :meth:`from_followed_conversations` depending on the user's settings.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        return self._client.get("/messages/my_feed", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def from_top_conversations(self, older_than=None, newer_than=None,
                               limit=None, threaded=None):
        """
        Returns messages from the current user's top conversations.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        return self._client.get("/messages/algo", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def from_followed_conversations(self, older_than=None, newer_than=None,
                                    limit=None, threaded=None):
        """
        Returns messages from users the current user follows, or groups
        the current user belongs to.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        return self._client.get("/messages/following", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def from_group(self, group_id, older_than=None, newer_than=None,
                   limit=None, threaded=None):
        """
        Returns messages from specific group, specified with `group_id`.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        path = "/messages/in_group/%d" % extract_id(group_id)
        return self._client.get(path, **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def sent(self, older_than=None, newer_than=None,
             limit=None, threaded=None):
        """
        Returns of the current user's sent messages.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        return self._client.get("/messages/sent", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def private(self, older_than=None, newer_than=None,
                limit=None, threaded=None):
        """
        Returns of the private messages received by the current user.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        return self._client.get("/messages/private", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def received(self, older_than=None, newer_than=None,
                 limit=None, threaded=None):
        """
        Returns messages received by the current user.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        return self._client.get("/messages/received", **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def in_thread(self, thread_id, older_than=None, newer_than=None,
                  limit=None, threaded=None):
        """
        Returns messages that belong to the thread identified by thread_id.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        path = "/messages/in_thread/%d" % extract_id(thread_id)
        return self._client.get(path, **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def from_user(self, user_id, older_than=None, newer_than=None,
                  limit=None, threaded=None):
        """
        Returns messages that were posted by the user identified by user_id.

        See the :meth:`all` method for a description of the keyword arguments.
        """
        path = "/messages/from_user/%d" % extract_id(user_id)
        return self._client.get(path, **self._argument_converter(
            older_than=older_than,
            newer_than=newer_than,
            limit=limit,
            threaded=threaded,
        ))

    def about_topic(self, topic_id):
        """
        Returns the messages about a topic
        """
        return self._client.get("/messages/about_topic/%d" % extract_id(topic_id))

    def find(self, message_id):
        """
        Returns the message identified by the given message_id.
        """
        return self._client.get("/messages/%d" % extract_id(message_id))

    def create(self, body, group_id=None, replied_to_id=None,
               direct_to_id=None, topics=[], broadcast=None,
               open_graph_object={}):
        """
        Posts a new message to Yammer. Returns the new message in the same
        format as the various message listing methods (:meth:`all`,
        :meth:`sent`, etc.).

        The following keyword arguments are supported:

        * ``group_id`` -- Send this message to the group identified by group_id.
        * ``replied_to_id`` -- This message is a reply to the message
          identified by replied_to_id.
        * ``direct_to_id`` -- Send this as a direct message to the user
          identified by direct_to_id.
        * ``topics`` -- A list of topics for the message. Topics should be
          given as strings. There cannot be more than 20 topics for one message.
        * ``broadcast`` -- Set this to True to send a broadcast message. Only
          network admins have permission to send broadcast messages.
        * ``open_graph_object`` -- A dict describing an open graph object to
          attach to the message. It supports the following keys:
           * ``url`` (*required*)
           * ``title``
           * ``image``
           * ``description``
           * ``object_type``
           * ``site_name``
           * ``fetch`` (set to ``True`` to derive other OG data from the URL)
           * ``meta`` (for custom structured data)
        """
        if len(topics) > 20:
            raise TooManyTopicsError("Too many topics, the maximum is 20")

        if len(open_graph_object) > 0 and "url" not in open_graph_object:
            raise InvalidOpenGraphObjectError("URL is required")

        return self._client.post("/messages", **self._argument_converter(
            body=body,
            group_id=group_id,
            replied_to_id=replied_to_id,
            direct_to_id=direct_to_id,
            topic=topics,
            broadcast=broadcast,
            og=open_graph_object,
        ))

    def delete(self, message_id):
        """
        Deletes the message identified by message_id.
        """
        return self._client.delete("/messages/%d" % extract_id(message_id))

    def like(self, message_id):
        """
        The current user likes the message identified by message_id.
        """
        return self._client.post(
            "/messages/liked_by/current",
            **self._argument_converter(
                message_id=message_id,
            )
        )

    def unlike(self, message_id):
        """
        Removes the current user's "like" from the message identified by
        message_id.
        """
        return self._client.delete(
            "/messages/liked_by/current",
            **self._argument_converter(
                message_id=message_id,
            )
        )

    def email(self, message_id):
        """
        Emails the message identified by message_id to the authenticated user.
        """
        return self._client.post("/messages/email", **self._argument_converter(
            message_id=message_id,
        ))



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from yampy.apis.utils import ArgumentConverter, flatten_dicts, \
                             stringify_booleans, none_filter
from yampy.errors import InvalidEducationRecordError, \
                         InvalidPreviousCompanyRecord
from yampy.models import extract_id


def education_argument_converter(arguments):
    result = arguments.copy()
    education = result.pop("education", None)

    if isinstance(education, dict):
        education = (education,)

    if education:
        record_format = "%(school)s,%(degree)s,%(description)s,"\
                        "%(start_year)s,%(end_year)s"
        try:
            result["education"] = [record_format % r for r in education]
        except KeyError as e:
            raise InvalidEducationRecordError(
                "Education record missing %s" % e.args
            )
    return result


def previous_companies_argument_converter(arguments):
    result = arguments.copy()
    previous_companies = result.pop("previous_companies", None)

    if isinstance(previous_companies, dict):
        previous_companies = (previous_companies, )

    if previous_companies:
        record_format = "%(company)s,%(position)s,%(description)s,"\
                        "%(start_year)s,%(end_year)s"
        try:
            result["previous_companies"] = [record_format % r for r in previous_companies]
        except KeyError as e:
            raise InvalidPreviousCompanyRecord(
                "Previous company record missing %s" % e.args
            )
    return result


class UsersAPI(object):
    """
    Provides an interface for accessing the user related endpoints of the
    Yammer API. You should not instantiate this class directly; use the
    :meth:`yampy.Yammer.users` method instead.
    """

    def __init__(self, client):
        """
        Initializes a new UsersAPI that will use the given client object
        to make HTTP requests.
        """
        self._client = client
        self._argument_converter = ArgumentConverter(
            education_argument_converter,
            previous_companies_argument_converter,
            flatten_dicts,
            stringify_booleans,
            none_filter,
        )

    def all(self, page=None, letter=None, sort_by=None, reverse=None):
        """
        Returns all the users in the current user's network.

        Customize the response using the keyword arguments:

        * page -- Enable pagination, and return the nth page of 50 users.
        * letter -- Only return users whose username begins with this letter.
        * sort_by -- Sort users by "messages" or "followers" (default order is
          alphabetical by username).
        * reverse -- Reverse sort order.
        """
        return self._client.get("/users", **self._argument_converter(
            page=page,
            letter=letter,
            sort_by=sort_by,
            reverse=reverse,
        ))

    def in_group(self, group_id, page=None):
        """
        Returns all the users belonging to the group identified by the given
        group_id.

        Use the page parameter to enable pagination and retrieve a specific
        page of users.
        """
        path = "/users/in_group/%d" % extract_id(group_id)
        return self._client.get(path, **self._argument_converter(
            page=page,
        ))

    def find_current(self, include_group_memberships=None,
                     include_followed_users=None, include_followed_tags=None):
        """
        Returns the current user.
        """
        return self._client.get("/users/current", **self._argument_converter(
            include_group_memberships=include_group_memberships,
            include_followed_users=include_followed_users,
            include_followed_tags=include_followed_tags,
        ))

    def find(self, user_id):
        """
        Returns the user identified by the given user_id.
        """
        return self._client.get(self._user_path(user_id))

    def find_by_email(self, email_address):
        """
        Returns the user identified by the given email_address.
        """
        return self._client.get("/users/by_email", **self._argument_converter(
            email=email_address,
        ))

    def create(self, email_address, full_name=None, job_title=None,
               location=None, im=None, work_telephone=None, work_extension=None,
               mobile_telephone=None, significant_other=None, kids_names=None,
               interests=None, summary=None, expertise=None, education=None,
               previous_companies=None):
        """
        Creates a new user.

        Most of the parameter names are self explanatory, and accept strings. A
        few expect specific formats:

        * ``im`` -- Provide instant messages details as a dict with ``provider``
          and ``username`` keys, e.g.
          ``{"provider": "gtalk", "username": "me@gmail.com"}``
        * ``education`` -- Provide education details as a list of dicts. Each
          dict should have the keys: ``school``, ``degree``, ``description``,
          ``start_year`` and ``end_year``.
        * ``previous_companies`` -- Provide previous employment details as a
          list of dicts. Each dict should have the keys: ``company``,
          ``position``, ``start_year``, ``end_year``.
        """

        return self._client.post("/users", **self._argument_converter(
            email=email_address,
            full_name=full_name,
            job_title=job_title,
            location=location,
            im=im,
            work_telephone=work_telephone,
            work_extension=work_extension,
            mobile_telephone=mobile_telephone,
            significant_other=significant_other,
            kids_names=kids_names,
            interests=interests,
            summary=summary,
            expertise=expertise,
            education=education,
            previous_companies=previous_companies,
        ))

    def update(self, user_id, full_name=None,
               job_title=None, location=None, im=None, work_telephone=None,
               work_extension=None, mobile_telephone=None,
               significant_other=None, kids_names=None, interests=None,
               summary=None, expertise=None, education=None,
               previous_companies=None):
        """
        Updates the user identified by the given user_id.

        For more information on parameter formats, see the :meth:`create`
        method.
        """

        path = self._user_path(user_id)
        return self._client.put(path, **self._argument_converter(
            full_name=full_name,
            job_title=job_title,
            location=location,
            im=im,
            work_telephone=work_telephone,
            work_extension=work_extension,
            mobile_telephone=mobile_telephone,
            significant_other=significant_other,
            kids_names=kids_names,
            interests=interests,
            summary=summary,
            expertise=expertise,
            education=education,
            previous_companies=previous_companies,
        ))

    def suspend(self, user_id):
        """
        Suspend the user identified by user_id.
        """
        return self._client.delete(self._user_path(user_id))

    def delete(self, user_id):
        """
        Delete the user identified by user_id.
        """
        return self._client.delete(self._user_path(user_id), delete="true")

    def _user_path(self, user_id):
        return "/users/%d" % extract_id(user_id)



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

"""
Constants, mostly concerned with building URLs
"""

DEFAULT_BASE_URL = "https://www.yammer.com/api/v1"
DEFAULT_OAUTH_BASE_URL = "https://www.yammer.com/oauth2"
DEFAULT_OAUTH_DIALOG_URL = "https://www.yammer.com/dialog/oauth"



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

from yampy.apis.utils import ArgumentConverter, IDExtractor, flatten_lists, \
                             flatten_dicts, stringify_booleans, none_filter


class RelationshipsAPI(object):
    """
    Provides an interface for accessing the relations related endpoints of the
    Yammer API. You should not instantiate this class directly; use the
    :meth:`yampy.Yammer.relationships` method instead.
    """

    def __init__(self, client):
        """
        Initializes a new RelationshipsAPI that will use the given ``client`` object
        to make HTTP requests.
        """
        self._client = client
        self._argument_converter = ArgumentConverter(
            IDExtractor(r"^(older|newer)_than|.*_id$"),
            flatten_lists,
            flatten_dicts,
            stringify_booleans,
            none_filter,
        )

    def all(self, user_id=None):
        """
        Returns the relationships for the current user or the user specified
        """
        return self._client.get("/relationships", **self._argument_converter(
            user_id=user_id,
        ))



# Copyright (c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR
# PURPOSE, MERCHANTABLITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

import os

from setuptools import setup
import yampy

VERSION = yampy.__version__

def readme():
    """ Load the contents of the README file """
    readme_path = os.path.join(os.path.dirname(__file__), "README.md")
    with open(readme_path, "r") as f:
        return f.read()

setup(
    name="yampy2",
    version=VERSION,
    description="The (un)official Python client for Yammer's API",
    author="Yammer",
    author_email="anthonyshaw@apache.org",
    long_description="Yammer API client",
    packages=["yampy", "yampy.apis"],
    install_requires=["requests"],
    license="Apache",
    url="http://github.com/tonybaloney/yampy2",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
    ],

    tests_require=["nose", "mock", "flask"],
    test_suite="nose.collector",
)
